----------------------------------------------------------------------------------
-- Create Date:    09:34:00 10/08/2012 
----------------------------------------------------------------------------------
library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity peak_display is
    Port (
        clk           : in  std_logic;
        reset         : in  std_logic;
        mode          : in  std_logic; -- 0: centroid. 1: moving average
        mean          : in  std_logic; -- 0: original. 1: compute the average value
        tick          : in  std_logic; -- tick when the peaks are computed and ready to be used (once per period)
        peak_centroid : in  std_logic_vector(RAMP_BITS-1 downto 0);
        peak_moving   : in  std_logic_vector(RAMP_BITS-1 downto 0);
        peak_output   : out std_logic_vector(RAMP_BITS-1 downto 0)
    );
end peak_display;

architecture Behavioral of peak_display is
    signal t : natural range 0 to DISPLAY_SAMPLES;
    signal peak_selected : std_logic_vector(RAMP_BITS-1 downto 0);
    signal quotient      : std_logic_vector(RAMP_BITS-1 downto 0);
    signal sum           : std_logic_vector(RAMP_BITS+DISPLAY_BITS-1 downto 0);
begin

process(clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            t        <= 0;
            sum      <= (others => '0');
            quotient <= (others => '0');
        else
            if tick = '1' then
                if t = DISPLAY_SAMPLES then
                    -- compute the average
                    quotient <= sum(RAMP_BITS+DISPLAY_BITS-1 downto DISPLAY_BITS);
                    sum      <= (others => '0');
                    t        <= 0;
                else
                    -- next iteration
                    sum <= std_logic_vector(unsigned(sum) + unsigned(peak_selected));
                    t   <= t + 1;
                end if;
            end if;
        end if;
    end if;
end process;

-- peak selection
with mode select
    peak_selected <= peak_centroid when '0', 
                     peak_moving   when others;

-- output selection
with mean select
    peak_output  <= peak_selected when '0',
                    quotient      when others;

end Behavioral;