----------------------------------------------------------------------------------
-- Create Date:    10:58:27 07/13/2012
----------------------------------------------------------------------------------
library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity uart_rx is
    generic (
        d_bits  : integer :=  8;  -- 0 00000001
        s_ticks : integer := 16
    );
    Port ( clk       : in  STD_LOGIC;
           reset     : in  STD_LOGIC;
           rx        : in  STD_LOGIC;
           baud_tick : in  STD_LOGIC;
           done_tick : out STD_LOGIC;
           dout      : out STD_LOGIC_VECTOR (d_bits-1 downto 0));
end uart_rx;

architecture Behavioral of uart_rx is
    type state_type is (idle, start, data, stop);
    signal state_reg, state_next: state_type;
    signal s_reg, s_next: unsigned(3 downto 0);
    signal n_reg, n_next: unsigned(2 downto 0);
    signal b_reg, b_next: std_logic_vector(d_bits-1 downto 0);

begin
    --FSMD register
    process(clk, reset)
    begin
        if(rising_edge(clk)) then
            if reset = '1' then
                state_reg <= idle;
                s_reg     <= (others=> '0');
                n_reg     <= (others=> '0');
                b_reg     <= (others=> '0');
            else
                state_reg <= state_next;
                s_reg     <= s_next;
                n_reg     <= n_next;
                b_reg     <= b_next;
            end if;
        end if;
    end process;

    --FSMD next logic
    process(state_reg, s_reg, n_reg, b_reg, rx, baud_tick)
    begin
        state_next    <= state_reg;
        s_next        <= s_reg;
        n_next        <= n_reg;
        b_next        <= b_reg;
        done_tick  <= '0';

        case state_reg is
            when idle =>
                if rx = '0' then
                    s_next <= (others=> '0');
                    state_next <= start;
                end if;

            when start =>
                if baud_tick = '1' then
                    if s_reg = 7 then
                        s_next     <= (others => '0');
                        n_next     <= (others => '0');
                        state_next <= data;
                    else
                        s_next <= s_reg + 1;
                    end if;
                end if;

            when data =>
                if baud_tick = '1' then
                    if s_reg = 15 then
                        s_next <= (others => '0');
                        b_next <= rx & b_reg(d_bits-1 downto 1);
                        if n_reg = (d_bits - 1) then
                            state_next <= stop;
                        else
                            n_next <= n_reg + 1;
                        end if;
                     else
                        s_next <= s_reg + 1;
                     end if;
                end if;

            when stop =>
                if baud_tick = '1' then
                    if s_reg = (s_ticks - 1) then
                        done_tick  <= '1';
                        state_next <= idle;
                    else
                        s_next <= s_reg + 1;
                    end if;
                end if;

        end case;
    end process;

    --FSMD output
    dout <= b_reg;

end Behavioral;
